const mysql = require('mysql');
class DatabaseConnection {
    constructor(config) {
        this.config = config;
        this.connection = null;
    }
    async open() {
        if (!this.connection) {
            this.connection = await mysql.createConnection(this.config);
            console.log('Database connected');
        }
    }
    async close() {
        if (this.connection) {
            await this.connection.end();
            console.log('Database connection closed');
        }
        this.connection = null;
    }
    async execute(query, callback) {
         try {
            await this.open();   
            this.connection.query(query, (err, rows) => {
                if (err) {
                    console.error('Error executing query:', err);
                    callback(0, rows);
                    this.closeConnection();
                    return;
                }    
                callback(1, rows);
                this.closeConnection();
            });
        } catch (err) {
            console.error('Error executing query:', err);
            callback(0, []);
        }
    }
    async closeConnection() {
        try {
            await this.close();
            console.log('Connection closed successfully.');
        } catch (closeErr) {
            console.error('Error closing connection:', closeErr.message || closeErr);
        }
    }
}

module.exports = DatabaseConnection;
